#include "pch.hpp"
#include "scoreline.hpp"
#include "MappedFile.hpp"
#include "SwrAddrDef.h"

#define MINIMAL_USE_PROCESSHEAPSTRING
#include "MinimalPath.hpp"

struct SCORELINE_HEADER {
	DWORD Magic;
	DWORD Version;
};

struct TRACKRECORD {
	BYTE ID1P;
	BYTE ID2P;
	BYTE WIN;
};

struct SCORELINE_V1 {
	SCORELINE_HEADER header;
	DWORD scoreLine[SWRCHAR_MAX][SWRCHAR_MAX][2];
	DWORD trCount;
	DWORD trTop;
	TRACKRECORD trackRecord[TRACKRECORD_MAX];
};

static DWORD s_scoreLineTR[SWRCHAR_MAX][SWRCHAR_MAX][2];
static DWORD (*s_pScoreLine)[SWRCHAR_MAX][SWRCHAR_MAX][2];

static Minimal::ProcessHeapStringT<char> s_srPath;
static MappedFile s_slFile;
static SCORELINE_V1 *s_sl;

static DWORD s_readMode;

static void ScoreLine_RefleshSLTR()
{
	ZeroMemory(s_scoreLineTR, sizeof s_scoreLineTR);
	DWORD range;
	switch(s_readMode) {
	case SCORELINE_READL30:  range = 30; break;
	case SCORELINE_READL50:  range = 50; break;
	case SCORELINE_READL100: range = 100;break;
	}
	DWORD count = range > s_sl->trCount? s_sl->trCount: range;
	DWORD bottom = s_sl->trTop - count;
	if(bottom < 0) bottom += TRACKRECORD_MAX;
	for(DWORD i = 0; i < count; ++i) {
		DWORD index = (bottom + i) % TRACKRECORD_MAX;
		TRACKRECORD &tr = s_sl->trackRecord[index];
		s_scoreLineTR[tr.ID1P][tr.ID2P][tr.WIN]++;
	}
}

void ScoreLine_Clear()
{
	ZeroMemory(s_sl->scoreLine, sizeof s_sl->scoreLine);
	s_sl->trTop = s_sl->trCount = 0;
}

void ScoreLine_Append(int index1P, int index2P, int win)
{
	s_sl->scoreLine[index1P][index2P][win]++;
	s_sl->trackRecord[s_sl->trTop].ID1P = index1P;
	s_sl->trackRecord[s_sl->trTop].ID2P = index2P;
	s_sl->trackRecord[s_sl->trTop].WIN  = win;

	s_sl->trTop = (s_sl->trTop + 1) % TRACKRECORD_MAX;
	if(s_sl->trCount < TRACKRECORD_MAX) {
		s_sl->trCount++;
	}
	if(s_pScoreLine == &s_scoreLineTR) ScoreLine_RefleshSLTR();
}

int ScoreLine_Read(int index1P, int index2P, int win)
{
	return (*s_pScoreLine)[index1P][index2P][win];
}

DWORD& ScoreLine_Reference(int index1P, int index2P, int win)
{
	return (*s_pScoreLine)[index1P][index2P][win];
}
int ScoreLine_GetReadMode()
{
	return s_readMode;
}

void ScoreLine_SetReadMode(int mode)
{
	s_readMode = mode;
	switch(mode) {
	case SCORELINE_READALL:
		s_pScoreLine = &s_sl->scoreLine;
		break;
	case SCORELINE_READL30:
	case SCORELINE_READL50:
	case SCORELINE_READL100:
		s_pScoreLine = &s_scoreLineTR;
		ScoreLine_RefleshSLTR();
		break;
	}
}

void TrackRecord_Clear()
{
	::ZeroMemory(s_sl->trackRecord, sizeof s_sl->trackRecord);
	::ZeroMemory(s_scoreLineTR, sizeof s_scoreLineTR);
	s_sl->trCount = s_sl->trTop = 0;

}

bool ScoreLine_Map(LPCSTR fileName, bool create)
{
	MappedFile slFile;
	SCORELINE_V1 *sl;

	if(!slFile.Open(fileName, 0, OPEN_EXISTING)) {
		if(!create || !slFile.Open(fileName, sizeof SCORELINE_V1)) return false;
		sl = (SCORELINE_V1 *)slFile.Map(0, sizeof SCORELINE_V1);
		ZeroMemory(sl, sizeof SCORELINE_V1);
		sl->header.Magic = 'HISL';
		sl->header.Version = 1;
	} else {
		if(slFile.GetSize() < sizeof SCORELINE_HEADER) return false;
		sl = (SCORELINE_V1 *)slFile.Map(0, sizeof SCORELINE_HEADER);
	}
	if(sl->header.Magic != 'HISL' || sl->header.Version != 1) {
		slFile.Unmap(sl);
		return false;
	}
	slFile.Unmap(sl);
	slFile.Remap(sizeof SCORELINE_V1);
	sl = (SCORELINE_V1 *)slFile.Map(0, 0);

	s_srPath = fileName;
	s_slFile.Unmap(s_sl);
	s_slFile = slFile;
	s_sl = sl;

	ScoreLine_SetReadMode(SCORELINE_READALL);
	return true;
}

LPCSTR ScoreLine_GetPath()
{
	return s_srPath;
}

void ScoreLine_Unmap()
{
	s_slFile.Unmap(s_sl);
	s_slFile.Close();
}
